#ifndef LSM_BT_INTERNAL
#define LSM_BT_INTERNAL

#include <stdint.h>

#include "lsm.h"

/**
 * Node inside a binary tree
 */
typedef struct lsm_bt_node {
  struct lsm_bt_node *left;
  struct lsm_bt_node *right;
  void *data;
  char key;
} lsm_bt_node;

/**
 * Initialize a new binary tree node
 *
 * @param ptr where to store newly allocated pointer
 * @param key key for the node
 * @param data data to store
 */
lsm_error lsm_bt_node_init(lsm_bt_node **ptr, const char key, void *data);

/**
 * Deallocate a single binary tree node
 */
void lsm_bt_node_free(lsm_bt_node *node);

/**
 * A binary tree implementation using char values as keys
 */
typedef struct lsm_bt {
  lsm_bt_node *root;
  uint8_t size;
} lsm_bt;

/**
 * Initialize a new binary tree
 *
 * @param ptr where to store newly allocated pointer
 */
lsm_error lsm_bt_init(lsm_bt **ptr);

/**
 * Deallocate an entire binary tree, including all its nodes
 */
void lsm_bt_free(lsm_bt *bt);

/**
 * Search for the data stored behind the given key.
 *
 * @param out pointer to store data pointer in
 * @param bt binary tree to search
 * @param key key to search
 */
lsm_error lsm_bt_search(void **out, lsm_bt *bt, char key);

/**
 * Insert a new data value into the tree with the given key.
 *
 * @param bt binary tree to insert into
 * @param key key to insert
 * @param data data to store
 */
lsm_error lsm_bt_insert(lsm_bt *bt, char key, void *data);

/**
 * Remove the given key from the binary tree. Ownership of the data pointer is
 * returned to the caller.
 *
 * @param out address to write data pointer to
 * @param bt binary tree to remove from
 * @param key key to remove
 */
lsm_error lsm_bt_remove(void **out, lsm_bt *bt, char key);

#endif
